.*: Assembler messages:
.*:1: Error: Illegal operands.*
.*:2: Error: A destination register must be supplied.*
.*:3: Error: Source and destination must be different.*
.*:10: Error: Illegal operands.*
.*:11: Error: A destination register must be supplied.*
.*:12: Error: Source and destination must be different.*
