#readelf: -A
#name: C6X array alignment attributes, directives 3
#as:
#source: attr-array-directive-3.s

Attribute Section: c6xabi
File Attributes
  Tag_ISA: C674x
  Tag_ABI_array_object_align_expected: 4-byte
