[^;]*: Assembler messages:
[^;]*:451: Warning: s suffix on comparison instruction is deprecated
[^;]*:451: Warning: s suffix on comparison instruction is deprecated
[^;]*:451: Warning: s suffix on comparison instruction is deprecated
[^;]*:451: Warning: s suffix on comparison instruction is deprecated
[^;]*:452: Warning: s suffix on comparison instruction is deprecated
[^;]*:452: Warning: s suffix on comparison instruction is deprecated
[^;]*:452: Warning: s suffix on comparison instruction is deprecated
[^;]*:452: Warning: s suffix on comparison instruction is deprecated
[^;]*:453: Warning: s suffix on comparison instruction is deprecated
[^;]*:453: Warning: s suffix on comparison instruction is deprecated
[^;]*:453: Warning: s suffix on comparison instruction is deprecated
[^;]*:453: Warning: s suffix on comparison instruction is deprecated
[^;]*:454: Warning: s suffix on comparison instruction is deprecated
[^;]*:454: Warning: s suffix on comparison instruction is deprecated
[^;]*:454: Warning: s suffix on comparison instruction is deprecated
[^;]*:454: Warning: s suffix on comparison instruction is deprecated
