.*: Assembler messages:
.*:9: Warning: Relaxed out-of-range branch into a jump
.*:14: Warning: Relaxed out-of-range branch into a jump
.*:19: Warning: Relaxed out-of-range branch into a jump
.*:24: Warning: Relaxed out-of-range branch into a jump
.*:28: Warning: Relaxed out-of-range branch into a jump
.*:33: Warning: Relaxed out-of-range branch into a jump
.*:37: Warning: Relaxed out-of-range branch into a jump
.*:42: Warning: Relaxed out-of-range branch into a jump
.*:46: Warning: Relaxed out-of-range branch into a jump
.*:51: Warning: Relaxed out-of-range branch into a jump
.*:55: Warning: Relaxed out-of-range branch into a jump
.*:62: Warning: Relaxed out-of-range branch into a jump
.*:68: Warning: Relaxed out-of-range branch into a jump
.*:73: Warning: Relaxed out-of-range branch into a jump
.*:79: Warning: Relaxed out-of-range branch into a jump
.*:85: Warning: Relaxed out-of-range branch into a jump
.*:96: Warning: Relaxed out-of-range branch into a jump
.*:101: Warning: Relaxed out-of-range branch into a jump
.*:106: Warning: Relaxed out-of-range branch into a jump
.*:111: Warning: Relaxed out-of-range branch into a jump
.*:116: Warning: Relaxed out-of-range branch into a jump
.*:121: Warning: Relaxed out-of-range branch into a jump
.*:126: Warning: Relaxed out-of-range branch into a jump
