.*: Assembler messages:
.*:6: Warning: Relaxed out-of-range branch into a jump
.*:9: Warning: Relaxed out-of-range branch into a jump
.*:7: Error: Branch out of range
.*:8: Error: Branch out of range
